@@ -1,8 +1,8 @@
 """Top-level package for Python Ember Mug."""
 from .mug import EmberMug
 
-__all__ = ('EmberMug',)
+__all__ = ("EmberMug",)
 
 __author__ = """Jesse Sopel"""
-__email__ = 'jesse.sopel@gmail.com'
-__version__ = '0.6.1'
+__email__ = "jesse.sopel@gmail.com"
+__version__ = "0.9.0"
