@@ -1,19 +1,20 @@
 """Classes for representing data from the mug."""
 from __future__ import annotations
 
-from dataclasses import asdict, dataclass, is_dataclass
-from datetime import datetime
+from dataclasses import asdict, dataclass, field, is_dataclass
 from functools import cached_property
-from typing import Any, NamedTuple
+from typing import TYPE_CHECKING, Any, NamedTuple
 
 from .consts import (
     ATTR_LABELS,
-    EMBER_CUP,
-    EMBER_TRAVEL_MUG,
-    EMBER_TRAVEL_MUG_SHORT,
+    DEVICE_MODEL_NAMES,
     EXTRA_ATTRS,
     INITIAL_ATTRS,
+    LIQUID_STATE_UNKNOWN,
     UPDATE_ATTRS,
+    DeviceColour,
+    DeviceModel,
+    DeviceType,
     LiquidState,
     TemperatureUnit,
     VolumeLevel,
@@ -21,6 +22,9 @@ from .consts import (
 from .formatting import format_led_colour, format_liquid_level, format_temp
 from .utils import bytes_to_little_int, decode_byte_string
 
+if TYPE_CHECKING:
+    from datetime import datetime
+
 
 class Change(NamedTuple):
     """Helper for storing changes to attributes."""
@@ -44,7 +48,7 @@ class Colour(NamedTuple):
 
     def as_hex(self) -> str:
         """Format colour array as hex string."""
-        return '#' + ''.join(f'{c:02x}' for c in self)[:6]
+        return "#" + "".join(f"{c:02x}" for c in self)[:6]
 
     def as_bytearray(self) -> bytearray:
         """Convert to byte array."""
@@ -71,7 +75,7 @@ class BatteryInfo:
         )
 
     def __str__(self) -> str:
-        """String representation for printing."""
+        """Format nicely for printing."""
         return f'{self.percent}%, {"" if self.on_charging_base else "not "}on charging base'
 
 
@@ -93,80 +97,80 @@ class MugFirmwareInfo:
         )
 
     def __str__(self) -> str:
-        """String representation for printing."""
-        return ', '.join(
+        """Format nicely for printing."""
+        return ", ".join(
             (
-                f'Version: {self.version}',
-                f'Hardware: {self.hardware}',
-                f'Bootloader: {self.bootloader}',
+                f"Version: {self.version}",
+                f"Hardware: {self.hardware}",
+                f"Bootloader: {self.bootloader}",
             ),
         )
 
 
-@dataclass(init=False)
-class Model:
+@dataclass
+class BaseModelInfo:
+    """Base class to declare properties as field."""
+
+    model: DeviceModel | None = None
+    colour: DeviceColour | None = None
+    name: str = field(init=False)
+    capacity: int | None = field(init=False)
+    device_type: DeviceType = field(init=False)
+
+
+@dataclass
+class ModelInfo(BaseModelInfo):
     """Model name and attributes based on mode."""
 
-    name: str
-    include_extra: bool = False
+    @cached_property  # type: ignore[misc]
+    def name(self) -> str:  # type: ignore[override]
+        """Get a human-readable name from model number."""
+        return DEVICE_MODEL_NAMES.get(
+            self.model or DeviceModel.UNKNOWN_DEVICE,
+            "Unknown Device",
+        )
 
-    def __init__(self, name: str, include_extra: bool = False) -> None:
-        """Override init to set long name for travel mug."""
-        if name == EMBER_TRAVEL_MUG_SHORT:
-            # Replace with full name for aesthetics
-            name = EMBER_TRAVEL_MUG
-        self.name = name
-        self.include_extra = include_extra
+    @cached_property  # type: ignore[misc]
+    def capacity(self) -> int | None:  # type: ignore[override]
+        """Determine capacity in mL based on model number."""
+        if self.model == DeviceModel.CUP_6_OZ:
+            return 178  # ml - 6oz
+        if self.model in (DeviceModel.MUG_1_10_OZ, DeviceModel.MUG_2_10_OZ):
+            return 295  # ml - 10oz
+        if self.model == DeviceModel.TRAVEL_MUG_12_OZ:
+            return 355  # ml - 12oz
+        if self.model in (DeviceModel.MUG_1_14_OZ, DeviceModel.MUG_2_14_OZ):
+            return 414  # ml - 14oz
+        if self.model == DeviceModel.TUMBLER_16_OZ:
+            return 473  # ml - 16oz
+        return None
+
+    @cached_property  # type: ignore[misc]
+    def device_type(self) -> DeviceType:  # type: ignore[override]
+        """Basic device type from model number."""
+        if self.model == DeviceModel.TRAVEL_MUG_12_OZ:
+            return DeviceType.TRAVEL_MUG
+        if self.model == DeviceModel.TUMBLER_16_OZ:
+            return DeviceType.TUMBLER
+        if self.model == DeviceModel.CUP_6_OZ:
+            return DeviceType.CUP
+        # This could be an unknown device, but fallback to mug
+        return DeviceType.MUG
 
     @cached_property
-    def is_cup(self) -> bool:
-        """Check if the model is a Cup."""
-        return self.name.startswith(EMBER_CUP)
-
-    @cached_property
-    def is_travel_mug(self) -> bool:
-        """Check if the model is a Travel mug."""
-        return self.name.startswith(EMBER_TRAVEL_MUG_SHORT)
-
-    @cached_property
-    def type(self) -> str:
-        """Model type as short string."""
-        if self.is_cup:
-            return "cup"
-        elif self.is_travel_mug:
-            return "travel_mug"
-        return "mug"
-
-    @cached_property
-    def attribute_labels(self) -> dict[str, str]:
-        """Calculated labels for includes attributes."""
-        all_attrs = self.initial_attributes | self.update_attributes | {'use_metric'}
-        return {attr: label for attr, label in ATTR_LABELS.items() if attr in all_attrs}
-
-    @cached_property
-    def initial_attributes(self) -> set[str]:
-        """Initial attributes based on model and extra."""
-        if self.include_extra is False:
-            return INITIAL_ATTRS - EXTRA_ATTRS
-        return INITIAL_ATTRS
-
-    @cached_property
-    def all_attributes(self) -> set[str]:
-        """All attributes."""
-        return self.initial_attributes | self.update_attributes
-
-    @cached_property
-    def update_attributes(self) -> set[str]:
+    def device_attributes(self) -> set[str]:
         """Attributes to update based on model and extra."""
-        attributes = UPDATE_ATTRS
-        if self.include_extra is False:
-            attributes = attributes - EXTRA_ATTRS
-        if self.is_cup:
-            # The Cup cannot be named
-            attributes = attributes - {'name'}
-        elif self.is_travel_mug:
+        attributes = EXTRA_ATTRS | INITIAL_ATTRS | UPDATE_ATTRS
+        unknown = (None, DeviceModel.UNKNOWN_DEVICE)
+        if self.model in unknown or self.device_type in (DeviceType.CUP, DeviceType.TUMBLER):
+            # The Cup and Tumbler cannot be named
+            attributes -= {"name"}
+        elif self.model in unknown or self.device_type == DeviceType.TRAVEL_MUG:
             # Tge Travel Mug does not have an LED colour, but has a volume attribute
-            attributes = (attributes - {'led_colour'}) | {'volume_level'}
+            attributes = (attributes - {"led_colour"}) | {"volume_level"}
+        if self.model != DeviceModel.TRAVEL_MUG_12_OZ:
+            # Only Travel mug has this attribute?
+            attributes -= {"battery_voltage"}
         return attributes
 
 
@@ -174,7 +178,7 @@ class Model:
 class MugMeta:
     """Meta data for mug."""
 
-    mug_id: str
+    mug_id: str  # unsure if this value is properly decoded
     serial_number: str
 
     @classmethod
@@ -186,8 +190,8 @@ class MugMeta:
         )
 
     def __str__(self) -> str:
-        """String representation for printing."""
-        return f'Mug ID: {self.mug_id}, Serial Number: {self.serial_number}'
+        """Format nicely for printing."""
+        return f"Mug ID: {self.mug_id}, Serial Number: {self.serial_number}"
 
 
 @dataclass
@@ -195,16 +199,17 @@ class MugData:
     """Class to store/display the state of the mug."""
 
     # Options
-    model: Model
+    model_info: ModelInfo
     use_metric: bool = True
+    debug: bool = False
 
     # Attributes
     name: str = ""
     meta: MugMeta | None = None
     battery: BatteryInfo | None = None
     firmware: MugFirmwareInfo | None = None
-    led_colour: Colour = Colour(255, 255, 255, 255)
-    liquid_state: LiquidState = LiquidState.UNKNOWN
+    led_colour: Colour = field(default_factory=lambda: Colour(255, 255, 255, 255))
+    liquid_state: LiquidState | None = None
     liquid_level: int = 0
     temperature_unit: TemperatureUnit = TemperatureUnit.CELSIUS
     current_temp: float = 0.0
@@ -218,8 +223,8 @@ class MugData:
     @property
     def meta_display(self) -> str:
         """Return Meta infor based on preference."""
-        if self.meta and not self.model.include_extra:
-            return f'Serial Number: {self.meta.serial_number}'
+        if self.meta and not self.debug:
+            return f"Serial Number: {self.meta.serial_number}"
         return str(self.meta)
 
     @property
@@ -230,7 +235,7 @@ class MugData:
     @property
     def liquid_state_display(self) -> str:
         """Human-readable liquid state."""
-        return self.liquid_state.label
+        return self.liquid_state.label if self.liquid_state else LIQUID_STATE_UNKNOWN
 
     @property
     def volume_level_display(self) -> str | None:
@@ -265,23 +270,29 @@ class MugData:
 
     def get_formatted_attr(self, attr: str) -> str | None:
         """Get the display value of a given attribute."""
-        if display_value := getattr(self, f'{attr}_display', None):
+        if display_value := getattr(self, f"{attr}_display", None):
             return display_value
         return getattr(self, attr)
 
     @property
     def formatted(self) -> dict[str, Any]:
         """Return human-readable names and values for all attributes for display."""
-        return {label: self.get_formatted_attr(attr) for attr, label in self.model.attribute_labels.items()}
+        all_attrs = self.model_info.device_attributes | {"use_metric"}
+        if not self.debug:
+            all_attrs -= EXTRA_ATTRS
+        return {label: self.get_formatted_attr(attr) for attr, label in ATTR_LABELS.items() if attr in all_attrs}
 
     def as_dict(self) -> dict[str, Any]:
         """Dump all attributes as dict for info/debugging."""
         data = {k: asdict(v) if is_dataclass(v) else v for k, v in asdict(self).items()}
+        all_attrs = self.model_info.device_attributes
+        if not self.debug:
+            all_attrs -= EXTRA_ATTRS
         data.update(
             {
-                f'{attr}_display': getattr(self, f'{attr}_display', None)
-                for attr in self.model.all_attributes
-                if hasattr(self, f'{attr}_display')
+                f"{attr}_display": getattr(self, f"{attr}_display", None)
+                for attr in all_attrs
+                if hasattr(self, f"{attr}_display")
             },
         )
         return data
