@@ -4,18 +4,17 @@ from __future__ import annotations
 import asyncio
 import contextlib
 import logging
-from collections.abc import AsyncIterator
 from datetime import datetime, timezone
 from enum import Enum
+from functools import cached_property
 from time import time
-from typing import Any, Callable, Literal
+from typing import TYPE_CHECKING, Any, Concatenate, Literal, ParamSpec, TypeVar
 
-from bleak import BleakClient, BleakError
-from bleak.backends.characteristic import BleakGATTCharacteristic
-from bleak.backends.device import BLEDevice
+from bleak import AdvertisementData, BleakClient, BleakError
 from bleak_retry_connector import establish_connection
 
 from .consts import (
+    INITIAL_ATTRS,
     IS_LINUX,
     MUG_NAME_REGEX,
     PUSH_EVENT_BATTERY_IDS,
@@ -23,21 +22,55 @@ from .consts import (
     MugCharacteristic,
     PushEvent,
     TemperatureUnit,
+    VolumeLevel,
 )
-from .data import BatteryInfo, Change, Colour, Model, MugData, MugFirmwareInfo, MugMeta
+from .data import BatteryInfo, Change, Colour, ModelInfo, MugData, MugFirmwareInfo, MugMeta
 from .utils import (
     bytes_to_big_int,
     bytes_to_little_int,
     decode_byte_string,
+    discover_services,
     encode_byte_string,
-    log_services,
+    get_model_info_from_advertiser_data,
     temp_from_bytes,
 )
 
+if TYPE_CHECKING:
+    from collections.abc import AsyncIterator, Awaitable, Callable
+
+    from bleak.backends.characteristic import BleakGATTCharacteristic
+    from bleak.backends.device import BLEDevice
+
+
 logger = logging.getLogger(__name__)
 
 DISCONNECT_DELAY = 120
 
+P = ParamSpec("P")
+T = TypeVar("T")
+
+
+def require_attribute(
+    attr_name: str,
+) -> Callable[[Callable[Concatenate[EmberMug, P], Awaitable[T]]], Callable[Concatenate[EmberMug, P], Awaitable[T]]]:
+    """Require an attribute to be available on the device."""
+
+    def decorator(
+        func: Callable[Concatenate[EmberMug, P], Awaitable[T]],
+    ) -> Callable[Concatenate[EmberMug, P], Awaitable[T]]:
+        """Inner decorator."""
+
+        async def wrapper(self: EmberMug, *args: P.args, **kwargs: P.kwargs) -> T:
+            if self.has_attribute(attr_name) is False:
+                raise NotImplementedError(
+                    f"The {self.data.model_info.device_type} " "does not have a name attribute",
+                )
+            return await func(self, *args, **kwargs)
+
+        return wrapper
+
+    return decorator
+
 
 class EmberMug:
     """Handle actual the actual mug connection and update states."""
@@ -45,17 +78,14 @@ class EmberMug:
     def __init__(
         self,
         ble_device: BLEDevice,
-        include_extra: bool = False,
+        model_info: ModelInfo,
         use_metric: bool = True,
         debug: bool = False,
         **kwargs: Any,
     ) -> None:
         """Initialize connection manager."""
         self.device = ble_device
-        self.data = MugData(
-            Model(ble_device.name or 'EMBER', include_extra),
-            use_metric=use_metric,
-        )
+        self.data = MugData(model_info, use_metric=use_metric, debug=debug)
 
         self.debug = debug
         self._connect_lock = asyncio.Lock()
@@ -70,10 +100,35 @@ class EmberMug:
         logger.debug("New mug connection initialized.")
         self.set_client_options(**kwargs)
 
-    def set_device(self, ble_device: BLEDevice) -> None:
-        """Set the ble device."""
-        logger.debug("Set new device from %s to %s", self.device, ble_device)
+    def ble_event_callback(self, ble_device: BLEDevice, advertisement_data: AdvertisementData) -> None:
+        """Update BLE Device and, if needed, model information."""
         self.device = ble_device
+        logger.debug("Set new device from %s to %s", self.device, ble_device)
+        if (
+            not self.data.model_info.model
+            and advertisement_data.manufacturer_data
+            and (model_info := get_model_info_from_advertiser_data(advertisement_data))
+        ):
+            logger.debug(
+                "Updated model info from advertisement data (%s) -> %s",
+                advertisement_data,
+                model_info,
+            )
+            self.data.model_info = model_info
+
+    @cached_property
+    def model_name(self) -> str | None:
+        """Shortcut to model name."""
+        return self.data.model_info.model.value if self.data.model_info.model else None
+
+    @property
+    def can_write(self) -> bool:
+        """Check if the mug can support write operations."""
+        return self.data.udsk is not None
+
+    def has_attribute(self, attribute: str) -> bool:
+        """Check whether the device has the given attribute."""
+        return attribute in self.data.model_info.device_attributes
 
     async def _ensure_connection(self) -> None:
         """Connect to mug."""
@@ -92,16 +147,15 @@ class EmberMug:
                 client = await establish_connection(
                     client_class=BleakClient,
                     device=self.device,
-                    name=f'{self.data.name} ({self.device.address})',
-                    use_services_cache=True,
-                    disconnected_callback=self._disconnect_callback,  # type: ignore
+                    name=f"{self.data.name} ({self.device.address})",
+                    disconnected_callback=self._disconnect_callback,
                     ble_device_callback=lambda: self.device,
                 )
                 if self.debug is True:
-                    log_services(client.services)
+                    await discover_services(client)
                 self._expected_disconnect = False
             except (asyncio.TimeoutError, BleakError) as error:
-                logger.error("%s: Failed to connect to the mug: %s", self.device, error)
+                logger.debug("%s: Failed to connect to the mug: %s", self.device, error)
                 raise error
             # Attempt to pair for good measure
             try:
@@ -111,25 +165,23 @@ class EmberMug:
             except NotImplementedError:
                 # workaround for Home Assistant ESPHome Proxy backend which does not allow pairing.
                 logger.warning(
-                    'Pairing not implemented. '
-                    'If your mug is still in pairing mode (blinking blue) tap the button on the bottom to exit.',
+                    "Pairing not implemented. "
+                    "If your mug is still in pairing mode (blinking blue) tap the button on the bottom to exit.",
                 )
             self._client = client
             await self.subscribe()
 
     async def _read(self, characteristic: MugCharacteristic) -> bytearray:
-        """Helper to read characteristic from Mug."""
-        if self._operation_lock.locked():
-            logger.debug("Operation already in progress. waiting for it to complete")
+        """Help read characteristic from Mug."""
+        self._check_operation_lock()
         async with self._operation_lock:
             data = await self._client.read_gatt_char(characteristic.uuid)
             logger.debug("Read attribute '%s' with value '%s'", characteristic, data)
             return data
 
     async def _write(self, characteristic: MugCharacteristic, data: bytearray) -> None:
-        """Helper to write characteristic to Mug."""
-        if self._operation_lock.locked():
-            logger.debug("Operation already in progress. Waiting for it to complete")
+        """Help write characteristic to Mug."""
+        self._check_operation_lock()
         async with self._operation_lock:
             await self._ensure_connection()
             try:
@@ -155,7 +207,7 @@ class EmberMug:
         if self._expected_disconnect:
             logger.debug("Disconnect callback called")
         else:
-            logger.warning("Unexpectedly disconnected")
+            logger.debug("Unexpectedly disconnected")
 
     def _fire_callbacks(self) -> None:
         """Fire the callbacks."""
@@ -163,6 +215,11 @@ class EmberMug:
         for callback in self._callbacks:
             callback(self.data)
 
+    def _check_operation_lock(self) -> None:
+        """Check and print message if lock occupied."""
+        if self._operation_lock.locked():
+            logger.debug("Operation already in progress. waiting for it to complete")
+
     def register_callback(self, callback: Callable[[MugData], None]) -> Callable[[], None]:
         """Register a callback to be called when the state changes."""
         if existing_unregister_callback := self._callbacks.get(callback):
@@ -178,6 +235,17 @@ class EmberMug:
         logger.debug("Registered callback: %s", callback)
         return unregister_callback
 
+    async def discover_services(self) -> dict[str, Any]:
+        """
+        Discover services for development or debugging.
+
+        Call discover_services with this client, ensuring the connection is active first.
+        """
+        self._check_operation_lock()
+        async with self._operation_lock:
+            await self._ensure_connection()
+            return await discover_services(self._client)
+
     async def get_meta(self) -> MugMeta:
         """Fetch Meta info from the mug (Serial number and ID)."""
         return MugMeta.from_bytes(await self._read(MugCharacteristic.MUG_ID))
@@ -186,13 +254,15 @@ class EmberMug:
         """Get Battery percent from mug gatt."""
         return BatteryInfo.from_bytes(await self._read(MugCharacteristic.BATTERY))
 
+    @require_attribute("led_colour")
     async def get_led_colour(self) -> Colour:
         """Get RGBA colours from mug gatt."""
-        return Colour.from_bytes(await self._read(MugCharacteristic.LED))
+        colour_data = await self._read(MugCharacteristic.LED)
+        return Colour(*bytearray(colour_data))
 
+    @require_attribute("led_colour")
     async def set_led_colour(self, colour: Colour) -> None:
         """Set new target temp for mug."""
-        colour = Colour(*colour[:3], 255)  # It always expects 255 for alpha
         await self._write(MugCharacteristic.LED, colour.as_bytearray())
         self.data.led_colour = colour
 
@@ -217,31 +287,54 @@ class EmberMug:
         liquid_level_bytes = await self._read(MugCharacteristic.LIQUID_LEVEL)
         return bytes_to_little_int(liquid_level_bytes)
 
+    @require_attribute("volume_level")
+    async def get_volume_level(self) -> VolumeLevel | None:
+        """Get volume level from mug gatt."""
+        volume_bytes = await self._read(MugCharacteristic.VOLUME)
+        volume_int = bytes_to_little_int(volume_bytes)
+        return VolumeLevel.from_state(volume_int)
+
+    @require_attribute("volume_level")
+    async def set_volume_level(self, volume: int | VolumeLevel) -> None:
+        """Set volume_level on Travel Mug."""
+        if not isinstance(volume, VolumeLevel) and isinstance(volume, int) and volume not in (0, 1, 2):
+            msg = "Volume level value should be 0, 1, 2 or a VolumeLevel enum"
+            raise ValueError(msg)
+        volume_level = volume if isinstance(volume, VolumeLevel) else VolumeLevel.from_state(volume)
+        await self._write(MugCharacteristic.VOLUME, bytearray([volume_level.state]))
+        self.data.volume_level = volume_level
+
     async def get_liquid_state(self) -> LiquidState:
         """Get liquid state from mug gatt."""
         liquid_state_bytes = await self._read(MugCharacteristic.LIQUID_STATE)
         state = bytes_to_little_int(liquid_state_bytes)
         return LiquidState(state)
 
+    @require_attribute("name")
     async def get_name(self) -> str:
         """Get mug name from gatt."""
         name_bytes: bytearray = await self._read(MugCharacteristic.MUG_NAME)
         return bytes(name_bytes).decode("utf8")
 
+    @require_attribute("name")
     async def set_name(self, name: str) -> None:
         """Assign new name to mug."""
         if MUG_NAME_REGEX.match(name) is None:
-            raise ValueError('Name cannot contain any special characters')
+            msg = "Name cannot contain any special characters and must be 16 characters or less"
+            raise ValueError(msg)
         await self._write(MugCharacteristic.MUG_NAME, bytearray(name.encode("utf8")))
         self.data.name = name
 
-    async def get_udsk(self) -> str:
+    async def get_udsk(self) -> str | None:
         """Get mug udsk from gatt."""
         try:
-            return decode_byte_string(await self._read(MugCharacteristic.UDSK))
-        except BleakError as e:
-            logger.debug('Unable to read UDSK: %s', e)
-        return ''
+            data = await self._read(MugCharacteristic.UDSK)
+            if data == bytearray([0] * 20):
+                return None
+            return decode_byte_string(data)
+        except (BleakError, ValueError) as e:
+            logger.debug("Unable to read UDSK: %s", e)
+        return None
 
     async def set_udsk(self, udsk: str) -> None:
         """Attempt to write udsk."""
@@ -253,8 +346,8 @@ class EmberMug:
         try:
             return decode_byte_string(await self._read(MugCharacteristic.DSK))
         except BleakError as e:
-            logger.debug('Unable to read DSK: %s', e)
-        return ''
+            logger.debug("Unable to read DSK: %s", e)
+        return ""
 
     async def get_temperature_unit(self) -> TemperatureUnit:
         """Get mug temp unit."""
@@ -285,7 +378,6 @@ class EmberMug:
         """Get date and time zone."""
         date_time_zone_bytes = await self._read(MugCharacteristic.DATE_TIME_AND_ZONE)
         time_value = bytes_to_big_int(date_time_zone_bytes[:4])
-        # offset = bytes_to_big_int(date_time_zone_bytes[4:])
         return datetime.fromtimestamp(time_value, timezone.utc) if time_value > 0 else None
 
     async def get_firmware(self) -> MugFirmwareInfo:
@@ -294,25 +386,27 @@ class EmberMug:
 
     async def update_initial(self) -> list[Change]:
         """Update attributes that don't normally change and don't need to be regularly updated."""
-        return await self._update_multiple(self.data.model.initial_attributes)
+        return await self._update_multiple(INITIAL_ATTRS)
 
     async def update_all(self) -> list[Change]:
         """Update all standard attributes."""
-        return await self._update_multiple(self.data.model.update_attributes)
+        return await self._update_multiple(
+            self.data.model_info.device_attributes - INITIAL_ATTRS,
+        )
 
     async def _update_multiple(self, attrs: set[str]) -> list[Change]:
-        """Helper to update a list of attributes from the mug."""
-        logger.debug('Updating the following attributes: %s', attrs)
+        """Update a list of attributes from the mug."""
+        logger.debug("Updating the following attributes: %s", attrs)
         await self._ensure_connection()
         changes = self.data.update_info(**{attr: await getattr(self, f"get_{attr}")() for attr in attrs})
         if changes:
             self._fire_callbacks()
-        logger.debug('Attributes updated: %s', changes)
+        logger.debug("Attributes updated: %s", changes)
         return changes
 
     async def update_queued_attributes(self) -> list[Change]:
         """Update all attributes in queue."""
-        logger.debug('Updating queued attributes: %s', self._queued_updates)
+        logger.debug("Updating queued attributes: %s", self._queued_updates)
         if not self._queued_updates:
             return []
         queued_updates = set(self._queued_updates)
@@ -331,7 +425,7 @@ class EmberMug:
             return
         self._latest_events[event_id] = now
 
-        logger.debug("Push event received from Mug (%s) - Data: %s.", event_id, data)
+        logger.debug("Push event received from %s (%s) - Data: %s.", self.model_name, event_id, data)
 
         # Check known IDs
         if event_id in PUSH_EVENT_BATTERY_IDS:
@@ -360,7 +454,7 @@ class EmberMug:
         elif event_id == PushEvent.BATTERY_VOLTAGE_STATE_CHANGED:
             self._queued_updates.add("battery_voltage")
         else:
-            logger.debug('Unknown even received %s', event_id)
+            logger.debug("Unknown event received %s", event_id)
 
     async def unsubscribe(self) -> None:
         """Unsubscribe from Mug notifications."""
@@ -380,13 +474,14 @@ class EmberMug:
 
     def set_client_options(self, **kwargs: str) -> None:
         """Update options in case they need to overriden in some cases."""
-        if kwargs.get('adapter') and IS_LINUX is False:
-            raise ValueError('The adapter option is only valid for the Linux BlueZ Backend.')
+        if kwargs.get("adapter") and IS_LINUX is False:
+            msg = "The adapter option is only valid for the Linux BlueZ Backend."
+            raise ValueError(msg)
         self._client_kwargs = {**kwargs}
 
     @contextlib.asynccontextmanager
     async def connection(self, **kwargs: str) -> AsyncIterator[EmberMug]:
-        """Helper for establishing a connection and automatically closing it."""
+        """Establish a connection and close automatically."""
         self.set_client_options(**kwargs)
         # This will happen automatically, but calling it now will give us immediate feedback
         await self._ensure_connection()
